//
//  pvlog.h
//  
//
//  Created by Brian Cornell on 10/9/17.
//
//

#ifndef pvlog_h
#define pvlog_h
#endif

#define VERMAJOR        1
#define VERMINOR        1
#define VERBUILD        16

#define TRUE            1
#define FALSE           0

#define WAIT_LOG_STOP   2               //Number of seconds to wait for logging to stop
#define WAIT_LOG_LOOP   1               //Number of seconds between process loop iterations in log mode

#define SYS_EPOCH_YEAR  1900            //Epoch for C library
#define LOG_EPOCH_YEAR  2000            //Epoch for logger HW

#define FMODE_LOG_DEV   "r+"            //If logging from device need r/w
#define FMODE_LOG_FIL   "r"             //Log file only needs to be read
#define FMODE_DAT       "w"             //Open mode for output data files

#define PROC_ID         0x3085          //Processor ID of logger HW
#define SIZ_PROC_ID     4               //Size of PROC_ID when printed to string w/o terminating NULL

#define CH_2_CHARGE     0x02
#define CH_4_BATTERY    0x04
#define CH_5_VOLTS      0x05

#define CHR_CHARGE      'C'
#define CHR_BATTERY     'B'
#define CHR_VOLTS       'V'
#define CHR_LOG_RAW     'r'
#define CHR_LOG_PROC    'p'
#define CHR_LOG_BEGIN   'b'

#define ADC_OFFSET      512
#define ADC_MVAMP       0.185
#define ADC_SCALE_V     6
#define ADC_DIFF_MAX    1
#define ADC_SZ_SAMPLE   3

#define SPL_RES_A       6               //6 samples/minute
#define SPL_RES_B       15              //15 RES_A samples (15 minutes)

#define SIZ_RSP_CMD     52              //Largest # of chars to process in logger handshake
#define SIZ_RSP_STRT    3               //# char to read in response to 'start logging' command
#define SIZ_ARGC_MIN    2               //Minimum # arguments
#define SIZ_ARGV_CMD    2               //Position for command
#define SIZ_ARGC_BASE   4               //Arguments required for 'log' command
#define SIZ_ARGV_BASE   (SIZ_ARGC_BASE - 1) //Base argument for 'log' command
#define SIZ_MAX_BASE    202             //Maximum size of user-supplied base path
#define SIZ_LOG_NAME    16              //Space required to add date & extension
#define SIZ_LOG_PATH    (SIZ_MAX_BASE + SIZ_LOG_NAME)
#define SIZ_BUF_IDS     (SIZ_PROC_ID + 2)   //Size of buffer to hold processor ID string
#define SIZ_REC_HDR     1               //# header records to read

#define STR_PRGNAME     "pvlog"         //Program name
#define STR_EXT_DAT     "csv"           //Data file extension
#define STR_EXT_LOG     "log"           //Log file extension
#define STR_HDR_RAW     "Sequence #,Date,Time,Data Type,Data Value"
#define STR_HDR_DMP     "Calculated"
#define STR_HDR_PRC     "Resolution,Date,Time,Data Type,Data Value"

#define FMT_DAT_PATH    "%s-%c-%4.4d%2.2d%2.2d.%s"
#define FMT_LOG_HDSK    "st\r\rso b\r\rid\r\r"  //Start logging handshake
//#define FMT_LOG_STRT    "sl\r\r"                //Start logging command (old)
#define FMT_LOG_STRT    "sd %02d%02d%02d%02d%02d%02d\r\rsl\r\r"
#define FMT_LOG_STOP    "st\r\r"                //Stop logging command
#define FMT_LOG_GID     "id\r\r"                //Get ID command
#define FMT_BUF_IDS     "%4.4X"                 //Format processor ID from Hex
#define FMT_ERR_FTRREC  "\nRead %u records but footer says %u."
#define FMT_ERR_SEQ     "\nSequence # mismatch:  expected %u, got %u."
#define FMT_ERR_STDERR  "\n%s:  %s, errno=%d."
#define FMT_TRM_SIGNAL  "\n%s terminating on signal %d.\n"
#define FMT_OUT_RESA    "\nA,%2.2d/%2.2d/%d,%2.2d:%2.2d:%2.2d,%c,%f,%f"
#define FMT_OUT_RESB    "\nB,%2.2d/%2.2d/%d,%2.2d:%2.2d:%2.2d,%c,%f,%f"
#define FMT_OUT_DUMP    "\n%u,%2.2d/%2.2d/%d,%2.2d:%2.2d:%2.2d,%c,%f,%f"
#define FMT_OUT_RAWTM   "\n%u,%2.2d/%2.2d/%d,%2.2d:%2.2d:%2.2d,"
#define FMT_OUT_RAWVL   "%c,%f"
#define FMT_PRG_DESC    "\nLogger parser for PV setup, v%d.%d build %d.  (c) 2017 Brian Cornell.\n"
#define FMT_PRG_SYNTX   "Syntax:  %s <file> (command).  Commands:"
#define FMT_PRG_CMDL    "\n\t%s - %s"
#define FMT_PRG_EXIT    "\n\n"
#define FMT_HDR_FIELDS  "%s,%s"
#define FMT_ERR_RTDMP   "\ndata:  %2X,%u.%u.%u,%u,%u,%u,%2X,%2X,%2X"
#define FMT_ERR_RECT    "\nData: 0x"
#define FMT_ERR_RECTV   " %.2X"

typedef struct {
    unsigned int seqnum;
    time_t time;
    float val_read;
    float val_calc;
    char pinc;
    char pinb;
} adc;

typedef struct {
    float read;
    float calc;
    char pinc;
} avg;

#define ADR_AVG_CHARGE  0
#define ADR_AVG_BATTERY 1
#define ADR_AVG_VOLTS   2

typedef struct {
    FILE *raw;                          //Stream to write raw data to
    FILE *prc;                          //Stream to write processed data to
    FILE *log;                          //Stream to write info/errors to
    int tm_mday;                        //Day of month conforming to time.h
} dats;

//Return codes
#define RC_SUCCESS      0
#define RC_NOARG        1
#define RC_BADARG       2
#define RC_LOGOFAIL     3
#define RC_LOGDFAIL     4
#define RC_LOGHFAIL     5
#define RC_LOGAFAIL     6
#define RC_LOGBFTR      7
#define RC_LOGNFTR      8
#define RC_LOGRECC      9
#define RC_DATCFAIL     10
#define RC_STOPFAIL     11
#define RC_SIGEXIT      12
#define RC_SIGERROR     13
#define RC_RECTYPE      14
#define RC_SEQNUMBR     15

const char *retCode[] = {
    "success",
    "no arguments",
    "bad or missing arguments",
    "couldn't open log device / file",
    "failed to start logging on device",
    "couldn't read / validate header",
    "couldn't read record",
    "bad footer",
    "no footer",
    "record count mismatch",
    "failed to open output files",
    "couldn't stop logger",
    "CTRL-C or TERM signal received",
    "error installing signal handler",
    "invalid record type",
    "bad sequence #"
};


//Commands
#define CMD_RAW         0
#define CMD_DUMP        1
#define CMD_PROC        2
#define CMD_LOG         3

#define SIZE_A_CMD      sizeof(Commands) / (sizeof(const char *) * 2)

const cmd Commands[] = {
    {"raw","raw dump of records - formatting only (default)"},
    {"dump","add power calculations but no averaging"},
    {"process","full analysis"},
    {"log","log data stream from <file> to <path>"}
};

float getVolts(int, int, int);
float getAmps(int, int, int, float);
unsigned int TimeSinceEpoch(logtm *);
adc * getSample(adc *, logag *, time_t *);
int RawRec(FILE *, logag *, time_t *);
void DumpRec(FILE *, logag *, time_t *);
void ProcessRec(FILE *, logag *, time_t *);
dats *openDAT(dats *, char *);
int startLog(FILE *);
int stopLog(FILE *, dats *);
void sigHandler(int);
time_t readHeader(FILE *);


//End file.